﻿using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Abagames.Util;
using Abagames.Util.Ssl;
using System.Runtime.InteropServices;

namespace Abagames.HundredMonkeys {
	public class Boss {
		private Random random = new Random();
		private Field field;
		private BossShape shape;
		private Vector3 pos, ppos;
		private int ticks;
		private SslbPool sslbs;
		private BulletPool bullets;
		private int barrageIndex, barrageTicks;

		public Boss(Frame frame, Field field, SslbPool sslbs, BulletPool bullets)
		{
			this.field = field;
			this.sslbs = sslbs;
			this.bullets = bullets;
			shape = new BossShape(200, frame);
			barrageIndex = -1;
		}

		public string[] barrageNames = {"nway", "away3way", "rocket", "rocket2"};
		public int[] barrageInterval = { 60, 60, 60, 60 };
		public int[] barrageDuration = { 500, 1000, 1000, 9999 };

		public unsafe void Update()
		{
			barrageTicks--;
			if (barrageTicks <= 0) {
				barrageIndex++;
				barrageTicks = barrageDuration[barrageIndex];
			}
			ppos = pos;
			pos.X = (float)Math.Sin(ticks * 0.02f) * field.Size.X * 0.5f;
			pos.Y = ((float)Math.Cos(ticks * 0.01f) * 0.2f + 0.7f) * field.Size.Y;
			shape.UpdatePosition(pos - ppos);
			ticks++;
			if (ticks % barrageInterval[barrageIndex] == 0) {
				Sslb sslb = sslbs.SslbDictionary[barrageNames[barrageIndex]];
				float[] args = new float[SslbExecutor.MaxArgsCount];
				for (int i = 0; i < sslb.DefaultArguments.Count; i++)
					args[i] = sslb.DefaultArguments[i].Calc(null);
				bullets.Add(sslb.CommandsId, args, pos.X, pos.Y, (float)Math.PI, 1.0f);
			}
		}

		public void Draw()
		{
			shape.Draw();
		}

	}

	public class BossShape : ActorPool<BossBlock>
	{
		private Random random = new Random();

		public BossShape(int n, Frame frame)
			: base(n)
		{
			shape = new CubeShape(frame, 1, 1, 0.5f);
			CreateBossShape();
		}

		private void CreateBossShape()
		{
			Vector3 p = new Vector3();
			Vector4 color = new Vector4();
			int bc = -1;
			for (int i = 0; i < 100; i++) {
				bc--;
				if (bc < 0) {
					p.X = (float)random.NextDouble() * 30.0f;
					p.Y = (float)random.NextDouble() * 40.0f;
					p.Z = (float)random.NextDouble() * 10.0f;
					bc = 5 + random.Next(10);
					color.X = CreateColor();
					color.Y = CreateColor();
					color.Z = CreateColor();
					color.W = 0.8f;
				}
				Add(p, color);
				p.X += (float)(random.NextDouble() * 2 - 1) * 5.0f;
				p.Y += (float)(random.NextDouble() * 2 - 1)* 5.0f;
				p.Z += (float)(random.NextDouble() * 2 - 1)* 1.0f;
				if (p.X < 0.1f)
					bc = 0;
			}
		}

		private float CreateColor()
		{
			if (random.Next(2) == 0)
				return (float)random.NextDouble() * 0.4f + 0.6f;
			else
				return (float)random.NextDouble() * 0.3f + 0.3f;
		}

		public void Add(Vector3 p, Vector4 c)
		{
			BossBlock a = new BossBlock();
			a.Pos = p;
			a.Scale = 3;
			a.Orientation = Quaternion.Identity;
			a.Color = c;
			Add(ref a);
			if (p.X > 0) {
				p.X *= -1;
				Add(p, c);
			}
		}

		public void UpdatePosition(Vector3 offset)
		{
			ForEach(delegate(ref BossBlock a) {
				a.Pos += offset;
			});
		}

		public override void Draw(ref BossBlock a)
		{
			Shape.AddInstance(a.Pos, a.Scale, a.Orientation, a.Color);
		}
	}

	[StructLayout(LayoutKind.Sequential)]
	public struct BossBlock {
		public Vector3 Pos;
		public float Scale;
		public Quaternion Orientation;
		public Vector4 Color;
	}
}
