﻿using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Abagames.Util;
using Abagames.Util.Ssl;
using System.Runtime.InteropServices;

namespace Abagames.HundredMonkeys {
	public class BulletPool : ActorPool<Bullet>, BulletsAddable {
		private Field field;
		private SslbPool sslbs;
		private ParticlePool particles;

		public BulletPool(int n, Frame frame, Field field, SslbPool sslbs, ParticlePool particles)
			: base(n)
		{
			this.field = field;
			this.sslbs = sslbs;
			this.particles = particles;
			shape = new CubeShape(frame, 1, 1, 0.5f);
		}

		public unsafe void Add(int commandsId, float* args, float x, float y, float angle, float speed)
		{
			Add(new SslbExecutor(commandsId, args, angle, speed), x, y);
		}

		public void Add(int commandsId, float[] args, float x, float y, float angle, float speed)
		{
			Add(new SslbExecutor(commandsId, args, angle, speed), x, y);
		}

		public void Add(SslbExecutor executor, float x, float y)
		{
			Bullet a = new Bullet();
			a.Executor = executor;
			a.Pos.X = x;
			a.Pos.Y = y;
			//a.Executor.Angle = angle;
			//a.Executor.Speed = speed;
			a.Orientation = Quaternion.Identity;
			a.Scale = 2;
			a.Color = new Vector4(1, 0, 0, 1);
			Add(ref a);
		}

		public override bool Update(ref Bullet a)
		{
			a.Executor.Update(sslbs, a.Pos.X, a.Pos.Y, this);
			float angle = a.Executor.Angle;
			float speed = a.Executor.Speed;
			a.Pos.X += (float)Math.Sin(angle) * speed;
			a.Pos.Y += (float)Math.Cos(angle) * speed;
			a.Ticks++;
			if (a.Ticks % 2 == 0)
				particles.Add(a.Pos, -speed, angle, a.Color, 2);
			return field.ContainsOuter(a.Pos) && a.Ticks < 300;
		}

		public bool CheckHit(Vector3 p, float d)
		{
			bool isHit = false;
			/*ForEach(delegate(ref Bullet a) {
				if (Vector3.Distance(a.Pos, p) < 0.5f)
					isHit = true;
			});*/
			for (int i = 0; i < actorCount; i++) {
				if (Vector3.Distance(Actors[i].Pos, p) < d) {
					isHit = true;
					Remove(i);
					i--;
				}
			}
			return isHit;
		}

		public override void Draw(ref Bullet a)
		{
			Shape.AddInstance(a.Pos, a.Scale, a.Orientation, a.Color);
		}
	}

	[StructLayout(LayoutKind.Sequential)]
	public struct Bullet {
		public Vector3 Pos;
		public float Scale;
		public Quaternion Orientation;
		public Vector4 Color;
		public SslbExecutor Executor;
		public int Ticks;
	}
}
