﻿using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Abagames.Util;
using Abagames.Util.Ssl;
using System.Runtime.InteropServices;

namespace Abagames.HundredMonkeys {
	public class Field {
		public Vector2 Size, SizeOuter;
		private FieldBlockPool fieldBlocks;
		private FieldBlockPool panels;

		public Field(Frame frame)
		{
			Size = new Vector2(60, 45);
			SizeOuter = new Vector2(70, 50);
			fieldBlocks = new FieldBlockPool(50, frame, this);
			for (int i = 0; i < 50; i++)
				fieldBlocks.Add();
			/*panels = new FieldBlockPool(50, frame, this);
			for (int x = 0; x < 10; x++)
				for (int y = 0; y < 100; y++)
					panels.Add();*/
		}

		public bool ContainsOuter(Vector3 p)
		{
			Vector2 p2 = new Vector2(p.X, p.Y);
			return MathUtil.Contains(p2, SizeOuter);
		}

		public void Update()
		{
			fieldBlocks.Update();
		}

		public void Draw()
		{
			fieldBlocks.Draw();
		}
	}

	public class FieldBlockPool : ActorPool<FieldBlock>
	{
		private Random random = new Random();
		private Field field;

		public FieldBlockPool(int n, Frame frame, Field field)
			: base(n)
		{
			this.field = field;
			shape = new CubeShape(frame, 1, 1, 5);
		}

		public void Add()
		{
			FieldBlock a = new FieldBlock();
			float s = 3 + 10 * (float)random.NextDouble();
			a.Pos.X = field.Size.X * (1 + (float)random.NextDouble() * 2);
			if (random.Next(2) == 0)
				a.Pos.X *= -1;
			a.Pos.Y = field.Size.Y * (-2 + random.Next(10));
			a.Pos.Z = s;
			a.Orientation = Quaternion.Identity;
			a.Color = new Vector4(1, 1, 1, 0.3f);
			a.Scale = s;
			Add(ref a);
		}

		public void AddPanel()
		{
			FieldBlock a = new FieldBlock();
			float s = 3 + 10 * (float)random.NextDouble();
			a.Pos.X = field.Size.X * (1 + (float)random.NextDouble() * 2);
			if (random.Next(2) == 0)
				a.Pos.X *= -1;
			a.Pos.Y = field.Size.Y * (-2 + random.Next(10));
			a.Pos.Z = s;
			a.Orientation = Quaternion.Identity;
			a.Color = new Vector4(1, 1, 1, 0.3f);
			a.Scale = s;
			Add(ref a);
		}

		public override bool Update(ref FieldBlock a)
		{
			a.Pos.Y -= 0.5f;
			if (a.Pos.Y < field.Size.Y * -2)
				a.Pos.Y += field.Size.Y * 10;
			return true;
		}

		public override void Draw(ref FieldBlock a)
		{
			Shape.AddInstance(a.Pos, a.Scale, a.Orientation, a.Color);
		}
	}

	[StructLayout(LayoutKind.Sequential)]
	public struct FieldBlock {
		public Vector3 Pos;
		public float Scale;
		public Quaternion Orientation;
		public Vector4 Color;
	}	
}
