﻿using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Abagames.Util;
using Abagames.Util.Ssl;
using System.Runtime.InteropServices;

namespace Abagames.HundredMonkeys {
	public class MonkeyPool : ActorPool<Monkey>
	{
		private Random random = new Random();
		private Field field;
		private BulletPool bullets;
		private ParticlePool particles;
		private int index;

		public MonkeyPool(int n, Frame frame, Field field, BulletPool bullets, ParticlePool particles)
			: base(n)
		{
			this.field = field;
			this.bullets = bullets;
			this.particles = particles;
			CubeListShape s = new CubeListShape(frame, 3);
			s.Add(new Vector3(0, 0, 0), new Vector3(1, 1, 1));
			s.Add(new Vector3(-2, 0, 0), new Vector3(0.5f, 0.5f, 0.5f));
			s.Add(new Vector3(2, 0, 0), new Vector3(0.5f, 0.5f, 0.5f));
			s.EndAddCube();
			shape = s;
		}

		public void Add()
		{
			Monkey a = new Monkey();
			a.Pos.X = ((float)random.NextDouble() * 2 - 1) * field.Size.X;
			a.Pos.Y = -(float)random.NextDouble() * field.Size.Y;
			a.Scale = 2;
			a.Orientation = Quaternion.Identity;
			a.Color = new Vector4(1, 1, 0, 0.8f);
			a.Ticks = random.Next(60);
			a.Index = index;
			index++;
			Add(ref a);
		}

		public override bool Update(ref Monkey a)
		{
			a.Pos += a.Vel;
			a.Ticks--;
			if (a.Ticks <= 0) {
				a.Vel.X = ((float)random.NextDouble() * 2 - 1) * 0.5f;
				a.Vel.Y = ((float)random.NextDouble() * 2 - 1) * 0.5f;
				a.Ticks = 60 + random.Next(30);
			}
			if ((a.Pos.X > field.Size.X && a.Vel.X > 0) ||
				(a.Pos.X < -field.Size.X && a.Vel.X < 0))
				a.Vel.X *= -1;
			if ((a.Pos.Y > 0 && a.Vel.Y > 0) ||
				(a.Pos.Y < -field.Size.Y && a.Vel.Y < 0))
				a.Vel.Y *= -1;
			float hitDist = 0.5f + actorCount * (0.5f / Actors.Length);
			if (actorCount <= 1)
				return true;
			if (bullets.CheckHit(a.Pos, hitDist)) {
				int pn = Actors.Length - actorCount;
				for (int i = 0; i < pn; i++)
					particles.Add(a.Pos, (float)random.NextDouble() * 3.0f, (float)random.NextDouble() * (float)Math.PI * 2, a.Color, 1);
				return false;
			}
			return true;
		}

		public override void Draw(ref Monkey a)
		{
			Shape.AddInstance(a.Pos, a.Scale, a.Orientation, a.Color);
		}

		public void DrawIndexes()
		{
			ForEach(delegate(ref Monkey a){
				Vector3 p = a.Pos;
				p.Y += 5;
				float s = 1 + 5.0f / (float)Math.Sqrt(actorCount);
				Letter.Add(a.Index + 1, p, s, Quaternion.CreateFromYawPitchRoll(0, 0.5f, 0), 0.8f);
				Letter.Draw();
			});
		}
	}

	[StructLayout(LayoutKind.Sequential)]
	public struct Monkey {
		public Vector3 Pos;
		public float Scale;
		public Quaternion Orientation;
		public Vector4 Color;
		public int Ticks;
		public Vector3 Vel;
		public int Index;
	}

}
