﻿using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Abagames.Util;
using Abagames.Util.Ssl;
using System.Runtime.InteropServices;

namespace Abagames.HundredMonkeys {
	public class ParticlePool : ActorPool<Particle> {
		private const float GroudZ = -1;
		private Random random = new Random();

		public ParticlePool(int n, Frame frame)
			: base(n)
		{
			shape = new CubeShape(frame, 0.2f, 0.2f, 1);
		}

		public void Add(Vector3 p, float speed, float angle, Vector4 c, float s)
		{
			Vector3 v = new Vector3();
			v.X = (float)Math.Sin(angle) * speed;
			v.Y = (float)Math.Cos(angle) * speed;
			Add(p, v, c, s);
		}

		public void Add(Vector3 p, Vector3 v, Vector4 c, float s)
		{
			Particle a = new Particle();
			a.Pos = p;
			a.Scale = 1;
			a.Orientation = Quaternion.Identity;
			a.Color = c;
			a.Vel.X = v.X * (0.75f + (float)random.NextDouble() * 0.5f);
			a.Vel.Y = v.Y * (0.75f + (float)random.NextDouble() * 0.5f);
			a.Vel.Z = v.Z * (0.75f + (float)random.NextDouble() * 0.5f);
			a.BaseScale = s;
			Add(ref a);
		}

		public override bool Update(ref Particle a)
		{
			a.Pos += a.Vel;
			a.Vel.Z -= 0.1f;
			a.BaseScale *= 0.99f;
			a.Vel *= 0.99f;
			a.Color.W *= 0.99f;
			if (a.Pos.Z < GroudZ) {
				a.Vel.Z *= -0.9f;
			}
			float vl = a.Vel.Length();
			a.Orientation.X = -a.Vel.Y;
			a.Orientation.Y = a.Vel.X;
			a.Orientation.Z = 0;
			a.Orientation.W = a.Vel.Z + vl;
			a.Orientation.Normalize();
			a.Scale = vl * a.BaseScale;
			return a.BaseScale > 0.5f;
		}

		public override void Draw(ref Particle a)
		{
			Shape.AddInstance(a.Pos, a.Scale, a.Orientation, a.Color);
		}
	}

	[StructLayout(LayoutKind.Sequential)]
	public struct Particle {
		public Vector3 Pos;
		public float Scale;
		public Quaternion Orientation;
		public Vector4 Color;
		public float BaseScale;
		public Vector3 Vel;
	}
}
