/*
 * $Id: Frame.cs,v 1.1.1.1 2009/08/22 11:57:01 kenta Exp $
 *
 * Copyright 2008 Kenta Cho. Some rights reserved.
 */
using System;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Content;

namespace Abagames.Util
{
    /// <summary>
    /// Effect, matrices and graphics.
    /// </summary>
    public abstract class Frame
    {
        public GraphicsDevice Gd;
        public Effect Effect;
        public Matrix ViewMatrix;
        public Matrix ProjMatrix;
        protected GraphicsDeviceManager graphics;
        protected ContentManager content;
        protected GameWindow window;
        protected Game game;
        protected String defaultTechniqueName;
        protected String effectFileName;

        public void SetParams(GraphicsDeviceManager graphics, ContentManager content, GameWindow window, Game game)
        {
            this.graphics = graphics;
            this.content = content;
            this.window = window;
            this.game = game;
        }

        public virtual void Initialize()
        {
        }

        public virtual void LoadContent()
        {
            Effect = content.Load<Effect>(effectFileName);
            Gd = graphics.GraphicsDevice;
            SetRenderState(Gd.RenderState);
        }

        public virtual void UnloadContent()
        {
        }

        public abstract void SetRenderState(RenderState renderState);
        public abstract void Update();
        public abstract void Draw();

        public virtual void Exit()
        {
            game.Exit();
        }

        public void BeginEffect()
        {
            BeginEffect(defaultTechniqueName);
        }

        public void BeginEffect(String techniqueName)
        {
            Effect.CurrentTechnique = Effect.Techniques[techniqueName];
            Effect.Begin();
            Effect.CurrentTechnique.Passes[0].Begin();
        }

        public void EndEffect()
        {
            Effect.CurrentTechnique.Passes[0].End();
            Effect.End();
        }

        public void ChangeTechnique(String techniqueName)
        {
            EndEffect();
            BeginEffect(techniqueName);
        }
    }
}
