/*
 * $Id: Letter.cs,v 1.1.1.1 2009/08/22 11:57:01 kenta Exp $
 *
 * Copyright 2008 Kenta Cho. Some rights reserved.
 */
using System;
using System.Collections.Generic;
using System.Runtime.InteropServices;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Content;
using Abagames.Util;

namespace Abagames.Util
{
    /// <summary>
    /// Letter renderer.
    /// </summary>
    public class Letter
    {
        private static LetterData[] letters;
        private static BarPool bars;

        public static void Initialize(ContentManager content, Frame frame)
        {
            List<List<string>> data = content.Load<List<List<string>>>(@"Xmls\Letters");
            letters = new LetterData[data.Count];
            int i = 0;
            foreach (List<string> bs in data)
            {
                letters[i] = new LetterData(bs);
                i++;
            }
            bars = new BarPool(512, frame);
        }

        public static void Draw()
        {
            bars.Draw();
            bars.Clear();
        }

        public static Vector3 Add(string s, Vector3 p, float scale, Quaternion orientation, float alpha)
        {
            Vector3 nl = new Vector3(scale * 2, 0, 0);
            nl = Vector3.Transform(nl, orientation);
            foreach (char c in s)
            {
                if (c != ' ')
                {
                    int ci = ConvertCharToInt(c);
                    letters[ci].Draw(bars, p, scale, orientation, alpha);
                }
                p += nl;
            }
            return p;
        }

        public static Vector3 Add(int n, Vector3 p, float scale, Quaternion orientation, float alpha)
        {
            Vector3 nl = new Vector3(-scale * 2, 0, 0);
            nl = Vector3.Transform(nl, orientation);
            for (; ; )
            {
                letters[n % 10].Draw(bars, p, scale, orientation, alpha);
                p += nl;
                n /= 10;
                if (n <= 0)
                    break;
            }
            return p;
        }

        public static Vector3 Add(int n, int digit, Vector3 p, float scale, Quaternion orientation, float alpha)
        {
            Vector3 nl = new Vector3(-scale * 2, 0, 0);
            nl = Vector3.Transform(nl, orientation);
            for (int i = 0; i < digit; i++)
            {
                letters[n % 10].Draw(bars, p, scale, orientation, alpha);
                p += nl;
                n /= 10;
            }
            return p;
        }

        private static int ConvertCharToInt(char c)
        {
            int idx = 0;
            if (c >= '0' && c <= '9')
                idx = c - '0';
            else if (c >= 'A' && c <= 'Z')
                idx = c - 'A' + 10;
            else if (c >= 'a' && c <= 'z')
                idx = c - 'a' + 10;
            else if (c == '.')
                idx = 36;
            else if (c == '-')
                idx = 38;
            else if (c == '+')
                idx = 39;
            else if (c == '_')
                idx = 37;
            else if (c == '!')
                idx = 42;
            else if (c == '/')
                idx = 43;
            return idx;
        }
    }

    public struct LetterData
    {
        private BarData[] data;

        public LetterData(List<string> bars)
        {
            data = new BarData[bars.Count];
            int i = 0;
            System.Globalization.NumberFormatInfo nfi = new System.Globalization.NumberFormatInfo();
            nfi.NumberDecimalSeparator = ".";
            foreach (string bar in bars)
            {
                string[] p = bar.Split(',');
                float x = float.Parse(p[0], nfi);
                float y = float.Parse(p[1], nfi);
                float w = float.Parse(p[2], nfi);
                float h = float.Parse(p[3], nfi);
                float a = -float.Parse(p[4], nfi) * (float)Math.PI / 180.0f;
                data[i].Offset.X = x;
                data[i].Offset.Y = y;
                data[i].Scale = w / 0.65f;
                data[i].Orientation = Quaternion.CreateFromYawPitchRoll(0, 0, a);
                i++;
            }
        }

        public void Draw(BarPool bars, Vector3 p, float scale, Quaternion orientation, float alpha)
        {
            foreach (BarData d in data)
            {
                bars.Add(p + Vector3.Transform(d.Offset, orientation) * scale, Quaternion.Concatenate(d.Orientation, orientation), d.Scale * scale * 0.1f, alpha);
            }
        }
    }

    public struct BarData
    {
        public Vector3 Offset;
        public float Scale;
        public Quaternion Orientation;
    }

    public class BarPool : ActorPool<Bar>
    {
        public BarPool(int n, Frame frame)
            : base(n)
        {
            shape = new CubeShape(frame, 6, 1, 1);
        }

        public void Add(Vector3 p, Quaternion o, float scale, float alpha)
        {
            Bar a = new Bar();
            a.Pos = p;
            a.Scale = scale;
            a.Orientation = o;
            a.Color.X = 1;
            a.Color.Y = 1;
            a.Color.Z = 1;
            a.Color.W = alpha;
            Add(ref a);
        }

        public override void Draw(ref Bar a)
        {
            Shape.AddInstance(a.Pos, a.Scale, a.Orientation, a.Color);
        }
    }

    [StructLayout(LayoutKind.Sequential)]
    public struct Bar
    {
        public Vector3 Pos;
        public float Scale;
        public Quaternion Orientation;
        public Vector4 Color;
    }
}
