/*
 * $Id: SectionStopwatch.cs,v 1.1.1.1 2009/08/22 11:57:02 kenta Exp $
 *
 * Copyright 2008 Kenta Cho. Some rights reserved.
 */
using System;
using System.Collections.Generic;
using System.Diagnostics;

namespace Abagames.Util
{
    /// <summary>
    /// Record a elapsed time of each section.
    /// </summary>
    public class SectionStopwatch
    {
        private const int maxSectionLength = 256;
        private long[] sectionTicks = new long[maxSectionLength];
        private float[] sectionTicksPerFrame = new float[maxSectionLength];
        private float[] sectionRates = new float[maxSectionLength];
        private int sectionIndex;
        private long prevTicks;
        private long totalTicks;
        private int frameCount;

        public void Start()
        {
            for (int i = 0; i < maxSectionLength; i++)
                sectionTicks[i] = 0;
            totalTicks = 0;
            frameCount = 0;
        }

        public void StartFrame()
        {
            sectionIndex = 0;
            StartSection();
        }

        public void StartSection()
        {
            prevTicks = Stopwatch.GetTimestamp();
        }

        public void NextSection()
        {
            long ct = Stopwatch.GetTimestamp();
            sectionTicks[sectionIndex] += ct - prevTicks;
            totalTicks += ct - prevTicks;
            prevTicks = ct;
            sectionIndex++;
        }

        public void EndFrame()
        {
            NextSection();
            for (int i = 0; i < sectionIndex; i++)
                sectionRates[i] = (float)sectionTicks[i] / totalTicks;
            frameCount++;
            for (int i = 0; i < sectionIndex; i++)
                sectionTicksPerFrame[i] = (float)sectionTicks[i] / frameCount;
        }
    }
}
