/*
 * $Id: Sound.cs,v 1.1.1.1 2009/08/22 11:57:02 kenta Exp $
 *
 * Copyright 2009 Kenta Cho. Some rights reserved.
 */
using System;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;

namespace Abagames.Util
{
    /// <summary>
    /// BGMs and SEs.
    /// </summary>
    public class Sound
    {
        private const int fadeoutInterval = 120;
#if XBOX360
        private const float baseVolume = 1.0f;
#else
        private const float baseVolume = 0.5f;
#endif
        private AudioEngine engine;
        private SoundBank soundBank;
        private WaveBank bgmWaveBank, seWaveBank;
        private Cue bgmCue;
        private AudioCategory bgmCategory, seCategory;
        private int fadeoutTicks;
        private AudioListener listener;
        private AudioEmitter emitter;

        public void Initialize()
        {
            engine = new AudioEngine(@"Content\Audio\Gtg.xgs");
            engine.Update();
            soundBank = new SoundBank(engine, @"Content\Audio\SoundBank.xsb");
            bgmWaveBank = new WaveBank(engine, @"Content\Audio\BgmWaveBank.xwb", 0, 16);
            seWaveBank = new WaveBank(engine, @"Content\Audio\SeWaveBank.xwb");
            bgmCategory = engine.GetCategory("Music");
            seCategory = engine.GetCategory("Default");
            bgmCategory.SetVolume(baseVolume);
            seCategory.SetVolume(baseVolume);
            fadeoutTicks = -1;
            bgmCue = null;
            listener = new AudioListener();
            emitter = new AudioEmitter();
        }

        public void Update()
        {
            if (fadeoutTicks > 0)
            {
                fadeoutTicks--;
                bgmCategory.SetVolume(baseVolume * (float)fadeoutTicks / fadeoutInterval);
                if (fadeoutTicks <= 0)
                    StopBgm();
            }
            engine.Update();
        }

        public void PlayBgm(string s)
        {
            StopBgm();
            bgmCue = soundBank.GetCue(s);
            bgmCategory.SetVolume(baseVolume);
            fadeoutTicks = -1;
            bgmCue.Play();
        }

        public void StopBgm()
        {
            if (bgmCue != null)
            {
                bgmCue.Stop(AudioStopOptions.AsAuthored);
                bgmCue = null;
            }
        }

        public void FadeoutBgm()
        {
            fadeoutTicks = fadeoutInterval;
        }

        public void PlaySe(string s)
        {
            soundBank.PlayCue(s);
        }

        public void PlaySe3D(string s, Vector3 p)
        {
            PlaySe3D(s, p, Vector3.Zero);
        }

        public void PlaySe3D(string s, Vector3 p, Vector3 v)
        {
            Cue c = soundBank.GetCue(s);
            emitter.Position = p;
            emitter.Velocity = v;
            c.Apply3D(listener, emitter);
            c.Play();
        }

        public Cue GetCue(string s)
        {
            return soundBank.GetCue(s);
        }

        public Vector3 ListenerPos
        {
            set
            {
                listener.Position = value;
            }

            get
            {
                return listener.Position;
            }
        }
    }
}
