﻿/*
 * $Id: SslbExecutor.cs,v 1.1.1.1 2009/08/22 11:57:02 kenta Exp $
 *
 * Copyright 2008 Kenta Cho. Some rights reserved.
 */
using System;
using System.Collections.Generic;

namespace Abagames.Util.Ssl
{
    public unsafe struct SslbExecutor
    {
        public const int MaxArgsCount = 16;
        private const int maxNestCount = 16;
        private fixed float arguments[MaxArgsCount];
        private int currentCounter;
        private int currentCommandsId;
        private int currentRepeatCount;
        //private float[,] argumentsStack;
        private fixed int counters[maxNestCount];
        private fixed int commandsIds[maxNestCount];
        private fixed int repeatCounts[maxNestCount];
        private int nestCount;
        private int waitTicks;
        private float angle, fireAngle;
        private float speed, fireSpeed;
        private float speedVel;
        private int speedVelTicks;

        public unsafe SslbExecutor(int commandsId, float* args, float angle, float speed)
        {
            currentCommandsId = commandsId;
            for (int i = 0; i < MaxArgsCount; i++)
            {
                fixed (float* eargs = arguments)
                {
                    eargs[i] = args[i];
                }
            }
            this.angle = fireAngle = angle;
            this.speed = fireSpeed = speed;
            currentCounter = 0;
            nestCount = 0;
            currentRepeatCount = 1;
            waitTicks = 0;
            speedVel = 0;
            speedVelTicks = 0;
        }

        public unsafe SslbExecutor(int commandsId, float[] args, float angle, float speed)
        {
            currentCommandsId = commandsId;
            for (int i = 0; i < args.Length; i++)
            {
                fixed (float* eargs = arguments)
                {
                    eargs[i] = args[i];
                }
            }
            this.angle = fireAngle = angle;
            this.speed = fireSpeed = speed;
            currentCounter = 0;
            nestCount = 0;
            currentRepeatCount = 1;
            waitTicks = 0;
            speedVel = 0;
            speedVelTicks = 0;
        }

        public bool Update(SslbPool sslbs, float x, float y, BulletsAddable ba)
        {
            waitTicks--;
            if (speedVelTicks > 0)
            {
                speedVelTicks--;
                speed += speedVel;
            }
            for (; ; )
            {
                if (waitTicks >= 0)
                    return true;
                if (currentCommandsId < 0)
                    return true;
                List<Command> commands = sslbs.CommandsList[currentCommandsId];
                while (currentCounter >= commands.Count)
                {
                    currentRepeatCount--;
                    if (currentRepeatCount <= 0)
                    {
                        if (nestCount <= 0)
                        {
                            currentCommandsId = -1;
                            return true;
                        }
                        else
                        {
                            nestCount--;
                            fixed (int* cnts = counters)
                            fixed (int* cids = commandsIds)
                            fixed (int* rpts = repeatCounts)
                            {
                                currentCounter = cnts[nestCount];
                                currentCommandsId = cids[nestCount];
                                currentRepeatCount = rpts[nestCount];
                            }
                            commands = sslbs.CommandsList[currentCommandsId];
                        }
                    }
                    else
                    {
                        currentCounter = 0;
                    }
                }
                Command c = commands[currentCounter];
                currentCounter++;
                fixed (float* args = arguments)
                {
                    if (c is Fire || c is ResetFire)
                    {
                        Fire f = (Fire)c;
                        if (c is ResetFire)
                        {
                            fireAngle = angle;
                            fireSpeed = speed;
                        }
                        if (f.Angle != null)
                            fireAngle += f.Angle.Calc(args) * (float)Math.PI / 180.0f;
                        if (f.Speed != null)
                            fireSpeed *= f.Speed.Calc(args);
                        ba.Add(f.CommandsId, args, x, y, fireAngle, fireSpeed);
                    }
                    else if (c is Repeat)
                    {
                        fixed (int* cnts = counters)
                        fixed (int* cids = commandsIds)
                        fixed (int* rpts = repeatCounts)
                        {
                            cnts[nestCount] = currentCounter;
                            cids[nestCount] = currentCommandsId;
                            rpts[nestCount] = currentRepeatCount;
                            nestCount++;
                            Repeat r = (Repeat)c;
                            currentCounter = 0;
                            currentCommandsId = r.CommandsId;
                            currentRepeatCount = (int)r.Count.Calc(args);
                        }
                    }
                    else if (c is Wait)
                    {
                        Wait w = (Wait)c;
                        waitTicks = (int)w.Term.Calc(args);
                    }
                    else if (c is Speed)
                    {
                        Speed s = (Speed)c;
                        float v = s.Value.Calc(args) * speed;
                        if (s.Term == null)
                        {
                            speed += v;
                        }
                        else
                        {
                            speedVelTicks = (int)s.Term.Calc(args);
                            speedVel = v / speedVelTicks;
                        }
                    }
                    /*else if (c is Call)
                    {
                        counters[nestCount] = currentCounter;
                        commandsIds[nestCount] = currentCommandsId;
                        nestCount++;
                        Call ca = (Call)c;
                    

                    }*/
                    else if (c is Vanish)
                    {
                        return false;
                    }
                }
            }
        }

        public float Angle
        {
            get
            {
                return angle;
            }
        }

        public float Speed
        {
            get
            {
                return speed;
            }
        }
    }

    public interface BulletsAddable
    {
		unsafe void Add(int commandsId, float* args, float x, float y, float angle, float speed);
		void Add(int commandsId, float[] args, float x, float y, float angle, float speed);
	}
}
