/*
 * $Id: IndentReader.cs,v 1.1.1.1 2009/08/22 11:57:02 kenta Exp $
 *
 * Copyright 2008 Kenta Cho. Some rights reserved.
 */
using System;
using System.Collections.Generic;
using System.Text;
using System.IO;

namespace Abagames.Util.Ssl
{
    /// <summary>
    /// 
    /// </summary>
    public class IndentReader
    {
        public delegate void BeginTopLevel(string[] strs);
        public delegate void EndTopLevel();
        public delegate void BeginIndent();
        public delegate void EndIndent();
        public delegate void ParseLine(string[] strs);

        public void Read(string filename, BeginTopLevel beginTopLevel, EndTopLevel endTopLevel, BeginIndent beginIndent, EndIndent endIndent, ParseLine parseLine)
        {
            System.Globalization.NumberFormatInfo nfi = new System.Globalization.NumberFormatInfo();
            nfi.NumberDecimalSeparator = ".";
            Stack<int> indentSpaceNums = new Stack<int>();
            int indentSpaceNum = 0;
            int indentLevel = 0;
            foreach (string line in GetLines(filename))
            {
                if (line.StartsWith("//"))
                    continue;
                string l = line.TrimStart();
                string[] strs = l.Split(' ');
                if (strs.Length <= 0)
                    continue;
                int isn = line.Length - l.Length;
                if (isn > indentSpaceNum)
                {
                    indentSpaceNums.Push(indentSpaceNum);
                    indentSpaceNum = isn;
                    indentLevel++;
                    beginIndent();
                }
                else if (isn < indentSpaceNum)
                {
                    while (isn < indentSpaceNum)
                    {
                        indentSpaceNum = indentSpaceNums.Pop();
                        indentLevel--;
                        endIndent();
                    }
                    if (indentLevel == 0)
                    {
                        endTopLevel();
                        continue;
                    }
                }
                if (indentLevel == 0)
                {
                    indentSpaceNum = 0;
                    indentSpaceNums.Push(indentSpaceNum);
                    beginTopLevel(strs);
                }
                else
                {
                    parseLine(strs);
                }
            }
            endTopLevel();
        }

        private static IEnumerable<string> GetLines(string filename)
        {
            using (StreamReader sr = new StreamReader(filename))
            {
                while (true)
                {
                    string line = sr.ReadLine();
                    if (line != null)
                        yield return line;
                    else
                        yield break;
                }
            }
        }
    }
}
