/*
 * $Id: SslImporter.cs,v 1.1.1.1 2009/08/22 11:57:02 kenta Exp $
 *
 * Copyright 2008 Kenta Cho. Some rights reserved.
 */
using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content.Pipeline;

namespace Abagames.Util.Ssl
{
    /// <summary>
    /// 
    /// </summary>
    public abstract class SslImporter<T> : ContentImporter<List<T>>
        where T : Ssl, new()
    {
        protected List<Command> commands;
        protected List<Command> nestedCommands;
        private List<T> ssls;
        private T ssl;
        private Stack<List<Command>> commandsStack;

        public override List<T> Import(string filename, ContentImporterContext context)
        {
            ssls = new List<T>();
            IndentReader reader = new IndentReader();
            reader.Read(filename, BeginTopLevel, EndTopLevel, BeginIndent, EndIndent, ParseLine);
            return ssls;
        }

        public void BeginTopLevel(string[] strs)
        {
            ssl = new T();
            ssl.Name = strs[0];
            List<Expression> args = new List<Expression>();
            for (int i = 1; i < strs.Length; i++)
                args.Add(new Expression(strs[i]));
            ssl.DefaultArguments = args;
            ssl.Commands = new List<Command>();
            nestedCommands = ssl.Commands;
            commandsStack = new Stack<List<Command>>();
            commands = null;
        }

        public void EndTopLevel()
        {
            ssls.Add(ssl);
        }

        public void BeginIndent()
        {
            commandsStack.Push(commands);
            commands = nestedCommands;
        }

        public void EndIndent()
        {
            commands = commandsStack.Pop();
        }

        public void ParseLine(string[] strs)
        {
            nestedCommands = null;
            switch (strs[0])
            {
                case "rep":
                    Repeat r = new Repeat();
                    r.Count = new Expression(strs[1]);
                    r.Commands = new List<Command>();
                    nestedCommands = r.Commands;
                    commands.Add(r);
                    break;
                case "wait":
                    Wait w = new Wait();
                    w.Term = new Expression(strs[1]);
                    commands.Add(w);
                    break;
                case "vns":
                    Vanish v = new Vanish();
                    commands.Add(v);
                    break;
                default:
                    ParseCommand(strs);
                    break;
            }
        }

        public abstract void ParseCommand(string[] strs);

        protected void AddCall(string[] strs)
        {
            Call c = new Call();
            c.Name = strs[0];
            List<Expression> args = new List<Expression>();
            for (int i = 1; i < strs.Length; i++)
                args.Add(new Expression(strs[i]));
            c.Arguments = args;
            commands.Add(c);
        }
    }
}
